@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2016-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2016-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -24,7 +24,6 @@ import html
 import collections
 
 import attr
-import sip
 from PyQt5.QtCore import (pyqtSlot, pyqtSignal, Qt, QTimer, QDir, QModelIndex,
                           QItemSelectionModel, QObject, QEventLoop)
 from PyQt5.QtWidgets import (QWidget, QGridLayout, QVBoxLayout, QLineEdit,
@@ -35,7 +34,8 @@ from qutebrowser.browser import downloads
 from qutebrowser.config import config
 from qutebrowser.utils import usertypes, log, utils, qtutils, objreg, message
 from qutebrowser.keyinput import modeman
-from qutebrowser.commands import cmdutils, cmdexc
+from qutebrowser.api import cmdutils
+from qutebrowser.qt import sip
 
 
 prompt_queue = None
@@ -384,14 +384,15 @@ class PromptContainer(QWidget):
         try:
             done = self._prompt.accept(value)
         except Error as e:
-            raise cmdexc.CommandError(str(e))
+            raise cmdutils.CommandError(str(e))
         if done:
             message.global_bridge.prompt_done.emit(self._prompt.KEY_MODE)
             question.done()
 
     @cmdutils.register(instance='prompt-container', scope='window',
                        modes=[usertypes.KeyMode.prompt], maxsplit=0)
-    def prompt_open_download(self, cmdline: str = None):
+    def prompt_open_download(self, cmdline: str = None,
+                             pdfjs: bool = False) -> None:
         """Immediately open a download.
 
         If no specific command is given, this will use the system's default
@@ -402,9 +403,10 @@ class PromptContainer(QWidget):
                      is expanded to the temporary file name. If no `{}` is
                      present, the filename is automatically appended to the
                      cmdline.
+            pdfjs: Open the download via PDF.js.
         """
         try:
-            self._prompt.download_open(cmdline)
+            self._prompt.download_open(cmdline, pdfjs=pdfjs)
         except UnsupportedOperationError:
             pass
 
@@ -422,6 +424,27 @@ class PromptContainer(QWidget):
         except UnsupportedOperationError:
             pass
 
+    @cmdutils.register(
+        instance='prompt-container', scope='window',
+        modes=[usertypes.KeyMode.prompt, usertypes.KeyMode.yesno])
+    def prompt_yank(self, sel=False):
+        """Yank URL to clipboard or primary selection.
+
+        Args:
+            sel: Use the primary selection instead of the clipboard.
+        """
+        question = self._prompt.question
+        if question.url is None:
+            message.error('No URL found.')
+            return
+        if sel and utils.supports_selection():
+            target = 'primary selection'
+        else:
+            sel = False
+            target = 'clipboard'
+        utils.set_clipboard(question.url, sel)
+        message.info("Yanked to {}: {}".format(target, question.url))
+
 
 class LineEdit(QLineEdit):
 
@@ -486,8 +509,8 @@ class _BasePrompt(QWidget):
         self._key_grid = QGridLayout()
         self._key_grid.setVerticalSpacing(0)
 
-        # The bindings are all in the 'prompt' mode, even for yesno prompts
-        all_bindings = config.key_instance.get_reverse_bindings_for('prompt')
+        all_bindings = config.key_instance.get_reverse_bindings_for(
+            self.KEY_MODE.name)
         labels = []
 
         for cmd, text in self._allowed_commands():
@@ -516,8 +539,10 @@ class _BasePrompt(QWidget):
     def accept(self, value=None):
         raise NotImplementedError
 
-    def download_open(self, _cmdline):
+    def download_open(self, cmdline, pdfjs):
         """Open the download directly if this is a download prompt."""
+        utils.unused(cmdline)
+        utils.unused(pdfjs)
         raise UnsupportedOperationError
 
     def item_focus(self, _which):
@@ -575,6 +600,8 @@ class FilenamePrompt(_BasePrompt):
         if config.val.prompt.filebrowser:
             self.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Preferred)
 
+        self._to_complete = ''
+
     @pyqtSlot(str)
     def _set_fileview_root(self, path, *, tabbed=False):
         """Set the root path for the file display."""
@@ -583,6 +610,9 @@ class FilenamePrompt(_BasePrompt):
             separators += os.altsep
 
         dirname = os.path.dirname(path)
+        basename = os.path.basename(path)
+        if not tabbed:
+            self._to_complete = ''
 
         try:
             if not path:
@@ -596,6 +626,7 @@ class FilenamePrompt(_BasePrompt):
             elif os.path.isdir(dirname) and not tabbed:
                 # Input like /foo/ba -> show /foo contents
                 path = dirname
+                self._to_complete = basename
             else:
                 return
         except OSError:
@@ -613,7 +644,11 @@ class FilenamePrompt(_BasePrompt):
             index: The QModelIndex of the selected element.
             clicked: Whether the element was clicked.
         """
-        path = os.path.normpath(self._file_model.filePath(index))
+        if index == QModelIndex():
+            path = os.path.join(self._file_model.rootPath(), self._to_complete)
+        else:
+            path = os.path.normpath(self._file_model.filePath(index))
+
         if clicked:
             path += os.sep
         else:
@@ -675,6 +710,7 @@ class FilenamePrompt(_BasePrompt):
         assert last_index.isValid()
 
         idx = selmodel.currentIndex()
+
         if not idx.isValid():
             # No item selected yet
             idx = last_index if which == 'prev' else first_index
@@ -688,10 +724,24 @@ class FilenamePrompt(_BasePrompt):
         if not idx.isValid():
             idx = last_index if which == 'prev' else first_index
 
+        idx = self._do_completion(idx, which)
+
         selmodel.setCurrentIndex(
             idx, QItemSelectionModel.ClearAndSelect | QItemSelectionModel.Rows)
         self._insert_path(idx, clicked=False)
 
+    def _do_completion(self, idx, which):
+        filename = self._file_model.fileName(idx)
+        while not filename.startswith(self._to_complete) and idx.isValid():
+            if which == 'prev':
+                idx = self._file_view.indexAbove(idx)
+            else:
+                assert which == 'next', which
+                idx = self._file_view.indexBelow(idx)
+            filename = self._file_model.fileName(idx)
+
+        return idx
+
     def _allowed_commands(self):
         return [('prompt-accept', 'Accept'), ('leave-mode', 'Abort')]
 
@@ -711,8 +761,13 @@ class DownloadFilenamePrompt(FilenamePrompt):
             self.question.answer = downloads.FileDownloadTarget(answer)
         return done
 
-    def download_open(self, cmdline):
-        self.question.answer = downloads.OpenFileDownloadTarget(cmdline)
+    def download_open(self, cmdline, pdfjs):
+        if pdfjs:
+            target = downloads.PDFJSDownloadTarget()
+        else:
+            target = downloads.OpenFileDownloadTarget(cmdline)
+
+        self.question.answer = target
         self.question.done()
         message.global_bridge.prompt_done.emit(self.KEY_MODE)
 
@@ -721,6 +776,8 @@ class DownloadFilenamePrompt(FilenamePrompt):
             ('prompt-accept', 'Accept'),
             ('leave-mode', 'Abort'),
             ('prompt-open-download', "Open download"),
+            ('prompt-open-download --pdfjs', "Open download via PDF.js"),
+            ('prompt-yank', "Yank URL"),
         ]
         return cmds
 
@@ -811,6 +868,7 @@ class YesNoPrompt(_BasePrompt):
         cmds = [
             ('prompt-accept yes', "Yes"),
             ('prompt-accept no', "No"),
+            ('prompt-yank', "Yank URL"),
         ]
 
         if self.question.default is not None:
