@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -101,9 +101,12 @@ def _generate_stylesheet():
         QWidget#StatusBar QLabel,
         QWidget#StatusBar QLineEdit {
             font: {{ conf.fonts.statusbar }};
-            background-color: {{ conf.colors.statusbar.normal.bg }};
             color: {{ conf.colors.statusbar.normal.fg }};
         }
+
+        QWidget#StatusBar {
+            background-color: {{ conf.colors.statusbar.normal.bg }};
+        }
     """
     for flag, option in flags:
         stylesheet += """
@@ -111,10 +114,13 @@ def _generate_stylesheet():
             QWidget#StatusBar[color_flags~="%s"] QLabel,
             QWidget#StatusBar[color_flags~="%s"] QLineEdit {
                 color: {{ conf.colors.%s }};
+            }
+
+            QWidget#StatusBar[color_flags~="%s"] {
                 background-color: {{ conf.colors.%s }};
             }
         """ % (flag, flag, flag,  # noqa: S001
-               option + '.fg', option + '.bg')
+               option + '.fg', flag, option + '.bg')
     return stylesheet
 
 
@@ -145,7 +151,7 @@ class StatusBar(QWidget):
     resized = pyqtSignal('QRect')
     moved = pyqtSignal('QPoint')
     _severity = None
-    _color_flags = []
+    _color_flags = None
 
     STYLESHEET = _generate_stylesheet()
 
@@ -182,26 +188,13 @@ class StatusBar(QWidget):
         self.cmd.hide_cmd.connect(self._hide_cmd_widget)
         self._hide_cmd_widget()
 
-        self.keystring = keystring.KeyString()
-        self._hbox.addWidget(self.keystring)
-
         self.url = url.UrlText()
-        self._hbox.addWidget(self.url)
-
         self.percentage = percentage.Percentage()
-        self._hbox.addWidget(self.percentage)
-
         self.backforward = backforward.Backforward()
-        self._hbox.addWidget(self.backforward)
-
         self.tabindex = tabindex.TabIndex()
-        self._hbox.addWidget(self.tabindex)
-
-        # We add a parent to Progress here because it calls self.show() based
-        # on some signals, and if that happens before it's added to the layout,
-        # it will quickly blink up as independent window.
+        self.keystring = keystring.KeyString()
         self.prog = progress.Progress(self)
-        self._hbox.addWidget(self.prog)
+        self._draw_widgets()
 
         config.instance.changed.connect(self._on_config_changed)
         QTimer.singleShot(0, self.maybe_hide)
@@ -215,6 +208,48 @@ class StatusBar(QWidget):
             self.maybe_hide()
         elif option == 'statusbar.padding':
             self._set_hbox_padding()
+        elif option == 'statusbar.widgets':
+            self._draw_widgets()
+
+    def _draw_widgets(self):
+        """Draw statusbar widgets."""
+        # Start with widgets hidden and show them when needed
+        for widget in [self.url, self.percentage,
+                       self.backforward, self.tabindex,
+                       self.keystring, self.prog]:
+            widget.hide()
+            self._hbox.removeWidget(widget)
+
+        tab = self._current_tab()
+
+        # Read the list and set widgets accordingly
+        for segment in config.val.statusbar.widgets:
+            if segment == 'url':
+                self._hbox.addWidget(self.url)
+                self.url.show()
+            elif segment == 'scroll':
+                self._hbox.addWidget(self.percentage)
+                self.percentage.show()
+            elif segment == 'scroll_raw':
+                self._hbox.addWidget(self.percentage)
+                self.percentage.raw = True
+                self.percentage.show()
+            elif segment == 'history':
+                self._hbox.addWidget(self.backforward)
+                self.backforward.enabled = True
+                if tab:
+                    self.backforward.on_tab_changed(tab)
+            elif segment == 'tabs':
+                self._hbox.addWidget(self.tabindex)
+                self.tabindex.show()
+            elif segment == 'keypress':
+                self._hbox.addWidget(self.keystring)
+                self.keystring.show()
+            elif segment == 'progress':
+                self._hbox.addWidget(self.prog)
+                self.prog.enabled = True
+                if tab:
+                    self.prog.on_tab_changed(tab)
 
     @pyqtSlot()
     def maybe_hide(self):
@@ -239,7 +274,7 @@ class StatusBar(QWidget):
         """Get the currently displayed tab."""
         window = objreg.get('tabbed-browser', scope='window',
                             window=self._win_id)
-        return window.currentWidget()
+        return window.widget.currentWidget()
 
     def set_mode_active(self, mode, val):
         """Setter for self.{insert,command,caret}_active.
@@ -260,17 +295,9 @@ class StatusBar(QWidget):
             log.statusbar.debug("Setting prompt flag to {}".format(val))
             self._color_flags.prompt = val
         elif mode == usertypes.KeyMode.caret:
-            tab = self._current_tab()
-            log.statusbar.debug("Setting caret flag - val {}, selection "
-                                "{}".format(val, tab.caret.selection_enabled))
-            if val:
-                if tab.caret.selection_enabled:
-                    self._set_mode_text("{} selection".format(mode.name))
-                    self._color_flags.caret = ColorFlags.CaretMode.selection
-                else:
-                    self._set_mode_text(mode.name)
-                    self._color_flags.caret = ColorFlags.CaretMode.on
-            else:
+            if not val:
+                # Turning on is handled in on_current_caret_selection_toggled
+                log.statusbar.debug("Setting caret mode off")
                 self._color_flags.caret = ColorFlags.CaretMode.off
         config.set_register_stylesheet(self, update=False)
 
@@ -346,7 +373,19 @@ class StatusBar(QWidget):
         self.percentage.on_tab_changed(tab)
         self.backforward.on_tab_changed(tab)
         self.maybe_hide()
-        assert tab.private == self._color_flags.private
+        assert tab.is_private == self._color_flags.private
+
+    @pyqtSlot(bool)
+    def on_caret_selection_toggled(self, selection):
+        """Update the statusbar when entering/leaving caret selection mode."""
+        log.statusbar.debug("Setting caret selection {}".format(selection))
+        if selection:
+            self._set_mode_text("caret selection")
+            self._color_flags.caret = ColorFlags.CaretMode.selection
+        else:
+            self._set_mode_text("caret")
+            self._color_flags.caret = ColorFlags.CaretMode.on
+        config.set_register_stylesheet(self, update=False)
 
     def resizeEvent(self, e):
         """Extend resizeEvent of QWidget to emit a resized signal afterwards.
