@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -25,8 +25,8 @@ from PyQt5.QtWidgets import (QLineEdit, QWidget, QHBoxLayout, QLabel,
 from PyQt5.QtGui import QValidator, QPainter
 
 from qutebrowser.config import config
-from qutebrowser.utils import utils, qtutils, log, usertypes
-from qutebrowser.misc import cmdhistory, objects
+from qutebrowser.utils import utils
+from qutebrowser.misc import cmdhistory
 
 
 class MinimalLineEditMixin:
@@ -94,7 +94,7 @@ class CommandLineEdit(QLineEdit):
         We use __ here to avoid accidentally overriding it in subclasses.
         """
         if new < self._promptlen:
-            self.setCursorPosition(self._promptlen)
+            self.cursorForward(self.hasSelectedText(), self._promptlen - new)
 
     def set_prompt(self, text):
         """Set the current prompt to text.
@@ -105,13 +105,6 @@ class CommandLineEdit(QLineEdit):
         self._validator.prompt = text
         self._promptlen = len(text)
 
-    def home(self, mark):
-        """Override home so it works properly with our cursor restriction."""
-        oldpos = self.cursorPosition()
-        self.setCursorPosition(self._promptlen)
-        if mark:
-            self.setSelection(self._promptlen, oldpos - self._promptlen)
-
 
 class _CommandValidator(QValidator):
 
@@ -260,22 +253,53 @@ class WrapperLayout(QLayout):
         self._widget = widget
         container.setFocusProxy(widget)
         widget.setParent(container)
-        if (qtutils.version_check('5.8.0', exact=True, compiled=False) and
-                objects.backend == usertypes.Backend.QtWebEngine and
-                container.window() and
-                container.window().windowHandle() and
-                not container.window().windowHandle().isActive()):
-            log.misc.debug("Calling QApplication::sync...")
-            # WORKAROUND for:
-            # https://bugreports.qt.io/browse/QTBUG-56652
-            # https://codereview.qt-project.org/#/c/176113/5//ALL,unified
-            QApplication.sync()
 
     def unwrap(self):
         self._widget.setParent(None)
         self._widget.deleteLater()
 
 
+class PseudoLayout(QLayout):
+
+    """A layout which isn't actually a real layout.
+
+    This is used to replace QWebEngineView's internal layout, as a WORKAROUND
+    for https://bugreports.qt.io/browse/QTBUG-68224 and other related issues.
+
+    This is partly inspired by https://codereview.qt-project.org/#/c/230894/
+    which does something similar as part of Qt.
+    """
+
+    def addItem(self, item):
+        assert self.parent() is not None
+        item.widget().setParent(self.parent())
+
+    def removeItem(self, item):
+        item.widget().setParent(None)
+
+    def count(self):
+        return 0
+
+    def itemAt(self, _pos):
+        return None
+
+    def widget(self):
+        return self.parent().render_widget()
+
+    def setGeometry(self, rect):
+        """Resize the render widget when the view is resized."""
+        widget = self.widget()
+        if widget is not None:
+            widget.setGeometry(rect)
+
+    def sizeHint(self):
+        """Make sure the view has the sizeHint of the render widget."""
+        widget = self.widget()
+        if widget is not None:
+            return widget.sizeHint()
+        return QSize()
+
+
 class FullscreenNotification(QLabel):
 
     """A label telling the user this page is now fullscreen."""
@@ -293,8 +317,6 @@ class FullscreenNotification(QLabel):
         bindings = all_bindings.get('fullscreen --leave')
         if bindings:
             key = bindings[0]
-            if utils.is_special_key(key):
-                key = key.strip('<>').capitalize()
             self.setText("Press {} to exit fullscreen.".format(key))
         else:
             self.setText("Page is now fullscreen.")
