@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -22,12 +22,9 @@
 import os
 import os.path
 import contextlib
-import traceback
-import mimetypes
 import html
 
 import jinja2
-import jinja2.exceptions
 from PyQt5.QtCore import QUrl
 
 from qutebrowser.utils import utils, urlutils, log
@@ -110,9 +107,8 @@ class Environment(jinja2.Environment):
         """Get a data: url for the broken qutebrowser logo."""
         data = utils.read_file(path, binary=True)
         filename = utils.resource_filename(path)
-        mimetype = mimetypes.guess_type(filename)
-        assert mimetype is not None, path
-        return urlutils.data_url(mimetype[0], data).toString()
+        mimetype = utils.guess_mimetype(filename)
+        return urlutils.data_url(mimetype, data).toString()
 
     def getattr(self, obj, attribute):
         """Override jinja's getattr() to be less clever.
@@ -125,14 +121,7 @@ class Environment(jinja2.Environment):
 
 def render(template, **kwargs):
     """Render the given template and pass the given arguments to it."""
-    try:
-        return environment.get_template(template).render(**kwargs)
-    except jinja2.exceptions.UndefinedError:
-        log.misc.exception("UndefinedError while rendering " + template)
-        err_path = os.path.join('html', 'undef_error.html')
-        err_template = utils.read_file(err_path)
-        tb = traceback.format_exc()
-        return err_template.format(pagename=template, traceback=tb)
+    return environment.get_template(template).render(**kwargs)
 
 
 environment = Environment()
