@@ -1,6 +1,6 @@
 # vim: ft=python fileencoding=utf-8 sts=4 sw=4 et:
 
-# Copyright 2014-2017 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
+# Copyright 2014-2019 Florian Bruhin (The Compiler) <mail@qutebrowser.org>
 #
 # This file is part of qutebrowser.
 #
@@ -29,9 +29,9 @@ import urllib.parse
 from PyQt5.QtCore import QUrl, QUrlQuery
 from PyQt5.QtNetwork import QHostInfo, QHostAddress, QNetworkProxy
 
+from qutebrowser.api import cmdutils
 from qutebrowser.config import config
 from qutebrowser.utils import log, qtutils, message, utils
-from qutebrowser.commands import cmdexc
 from qutebrowser.browser.network import pac
 
 
@@ -39,6 +39,21 @@ from qutebrowser.browser.network import pac
 # https://github.com/qutebrowser/qutebrowser/issues/108
 
 
+# URL schemes supported by QtWebEngine
+WEBENGINE_SCHEMES = [
+    'about',
+    'data',
+    'file',
+    'filesystem',
+    'ftp',
+    'http',
+    'https',
+    'javascript',
+    'ws',
+    'wss',
+]
+
+
 class InvalidUrlError(ValueError):
 
     """Error raised if a function got an invalid URL.
@@ -101,7 +116,14 @@ def _get_search_url(txt):
     if engine is None:
         engine = 'DEFAULT'
     template = config.val.url.searchengines[engine]
-    url = qurl_from_user_input(template.format(urllib.parse.quote(term)))
+    quoted_term = urllib.parse.quote(term, safe='')
+    url = qurl_from_user_input(template.format(quoted_term))
+
+    if config.val.url.open_base_url and term in config.val.url.searchengines:
+        url = qurl_from_user_input(config.val.url.searchengines[term])
+        url.setPath(None)
+        url.setFragment(None)
+        url.setQuery(None)
     qtutils.ensure_valid(url)
     return url
 
@@ -339,7 +361,7 @@ def invalid_url_error(url, action):
 def raise_cmdexc_if_invalid(url):
     """Check if the given QUrl is invalid, and if so, raise a CommandError."""
     if not url.isValid():
-        raise cmdexc.CommandError(get_errstring(url))
+        raise cmdutils.CommandError(get_errstring(url))
 
 
 def get_path_if_valid(pathstr, cwd=None, relative=False, check_exists=False):
@@ -537,7 +559,7 @@ def incdec_number(url, incdec, count=1, segments=None):
         incdec: Either 'increment' or 'decrement'
         count: The number to increment or decrement by
         segments: A set of URL segments to search. Valid segments are:
-                  'host', 'path', 'query', 'anchor'.
+                  'host', 'port', 'path', 'query', 'anchor'.
                   Default: {'path', 'query'}
 
     Return:
@@ -550,7 +572,7 @@ def incdec_number(url, incdec, count=1, segments=None):
 
     if segments is None:
         segments = {'path', 'query'}
-    valid_segments = {'host', 'path', 'query', 'anchor'}
+    valid_segments = {'host', 'port', 'path', 'query', 'anchor'}
     if segments - valid_segments:
         extra_elements = segments - valid_segments
         raise IncDecError("Invalid segments: {}".format(
@@ -561,6 +583,8 @@ def incdec_number(url, incdec, count=1, segments=None):
     # Order as they appear in a URL
     segment_modifiers = [
         ('host', url.host, url.setHost),
+        ('port', lambda: str(url.port()) if url.port() > 0 else '',
+         lambda x: url.setPort(int(x))),
         ('path', url.path, url.setPath),
         ('query', url.query, url.setQuery),
         ('anchor', url.fragment, url.setFragment),
